(function() {
    'use strict';

    angular.module('weChatApp')
        .directive('doughnutChart', ['AppConstants', DoughnutChart])
    ;

    function DoughnutChart(appConstants) {
        var ONE_HUNDRED = 100;

        return {
            template: '<canvas></canvas>',
            restrict: 'E',
            replace: true,
            transclude: true,
            scope: {
                name: '@',
                width: '@',
                height: '@',
                color1: '@',
                color2: '@',
                percentage: '='
            },
            link: function(scope, element) {
                if(scope.width) {
                    element.css('width', scope.width);
                }
                if(scope.height) {
                    element.css('height', scope.height);
                }

                var gradient = element[0].getContext("2d").createLinearGradient(0, 0, 0, 200);
                gradient.addColorStop(0, scope.color1);
                gradient.addColorStop(1, scope.color2);

                var adjustedPercentage = Math.min(parseInt(scope.percentage), ONE_HUNDRED);
                var data = {
                    datasets: [{
                        data: [adjustedPercentage, ONE_HUNDRED - adjustedPercentage],
                        backgroundColor: [
                            gradient,
                            "rgb(205,205,205)"
                        ]
                    }]
                };

                var options = {
                    cutoutPercentage: 80,
                    responsive: false,
                    events: [],
                    customLabel: 'label',
                    customLabelColor: '#666666',
                    centerLabel: {
                        text1: adjustedPercentage + '%',
                        text1Color: 'rgb(102,102,102)'
                    } // enable center label plugin
                };

                var doughnutChart = new Chart(element, {
                    type: 'doughnut',
                    data: data,
                    options: options
                });

                scope.$on(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, function(event, params) {
                    if(params.name === scope.name) {
                        adjustedPercentage = Math.min(parseInt(params.data), ONE_HUNDRED);
                        doughnutChart.data.datasets[0].data = [adjustedPercentage, ONE_HUNDRED - adjustedPercentage];
                        doughnutChart.options.centerLabel.text1 = adjustedPercentage + '%';
                        doughnutChart.update();
                    }
                });
            }
        };
    }
}());